/**
 * @license
 * Copyright 2019 Google Inc. All Rights Reserved.
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * =============================================================================
 */
/**
 * Get a key in an object by its value. This is used to get protobuf enum value
 * from index.
 *
 * @param object
 * @param value
 */
export declare function getEnumKeyFromValue(object: any, value: number): string;
/**
 * Read SavedModel proto message from path.
 *
 * @param path Path to SavedModel folder.
 */
export declare function readSavedModelProto(path: string): Promise<any>;
/**
 * Inspect the MetaGraphs of the SavedModel from the provided path.
 *
 * @param path Path to SavedModel folder.
 */
export declare function getMetaGraphsFromSavedModel(path: string): Promise<MetaGraphInfo[]>;
/**
 * Interface for inspected SavedModel MetaGraph info..
 */
export interface MetaGraphInfo {
    tags: string[];
    signatureDefs: SignatureDefInfo;
}
/**
 * Interface for inspected SavedModel SignatureDef info..
 */
export interface SignatureDefInfo {
    [key: string]: {
        inputs: {
            [key: string]: SavedModelTensorInfo;
        };
        outputs: {
            [key: string]: SavedModelTensorInfo;
        };
    };
}
/**
 * Interface for inspected SavedModel signature input/output Tensor info..
 */
export interface SavedModelTensorInfo {
    dtype: string;
    shape: number[];
    name: string;
}
